react-create-helper
===============
CLI tool to help with creating components, reducers and actions.

Features
--------
#####Create react-* with predefined templates
- Components
- Redux actions
- Redux reducers


### Install

```$bash
npm i -g react-create-helper
```
Options
-------
Add new string to `package.json` with name `"react-create-helper"` <br>
- `base` (string) - Base dir for components, actions, and reducers. Default - project root (`./`)
- `actionsDir` (string) - Base dir for actions, extends `base`. Default - project root (`./`)
- `reducersDir` (string) - Base dir for reducers, extends `base`. Default - project root (`./`)
- `styleExtension` (string) - Extensions for generated style file. Default - `css`
- `componentName.prepend` (string) - Prefix for generated component. Default - `undefined`
- `componentName.append` (string) - Suffix for generated component. Default - `undefined`
- `template.component` (string) - Path for your own component template. Default - `undefined`
- `template.reducer` (string) - Path for your own reducer template. Default - `undefined`
- `template.action` (string) - Path for your own action template. Default - `undefined`


## Example options

```$js
"react-create-helper": {
    "base": "./src",
    "actionsDir": "Actions",
    "reducersDir": "Reducers",
    "componentName": {
      "prepend": "My",
      "append": "Component"
    },
    "styleExtension": "scss"
  }
```


### Basic usage
**Cli options**
- `-c` generate component (default)
- `-r` generate reducer
- `-a` generate action
- `-h` help (options and usage examples)

```$bash
rch Header
```

output

```
Header
├── Header.js
├── Header.scss
└── index.js
```
Also you can specify path like `rch my/path/Header`

##### Generate action & reducers

```bash
$ rch -a ExampleAction
$ rch -r ExampleReducer
```
***This will generate something like this***

```
src
  ├── Actions
  │   └── ExampleAction.js
  └── Reducers
      └── ExampleReducer.js
```


## Define own templates

```$js
"react-create-helper": {
    "template": {
      "component": "rch_templates/component.js",
      "reducer": "rch_templates/reducer.js",
      "action": "rch_templates/action.js",
    },
  }
```

######Base template
`{basename}` is the name you provided with `rch -c {basename}`

`./rch_templates/component.js`
```js

module.exports = ({basename}) =>
`import React, { Component } from 'react';
import { connect } from 'react-redux';

class ${basename} extends Component {
  render() {
    return (
      <div></div>
	  )
  }
}

const mapStateToProps = (state) => {
  return {
    
  }
};
const mapDispatchToProps = (dispatch) => {
  return {
    exampleName: (exampleProps) => {
      dispatch(exampleActionName)
    }
  }
};

export default connect(mapStateToProps, mapDispatchToProps)(${basename});
`;

```

The same for action, and reducer.

######Have fun