#! /usr/bin/env node

const fs = require("fs"),
  path = require("path"),
  program = require("commander"),
  chalk = require('chalk'),
  mkdirSync = require('fs-force-mkdir-sync');
  config = require(path.resolve(process.cwd(), 'package.json'))['react-create-helper'];

const TYPE = {
  ACTION: "ACTION",
  REDUCER: "REDUCER"
};

function run(name, options) {

  let stylesExt = options.styles || "css";
  let sourceName = name;
  let folderProvided = path.dirname(sourceName);
  let folderInput = folderProvided !== "." ? folderProvided : false;

  if(folderInput)
    sourceName = path.basename(sourceName);

  if(typeof config !== "undefined"){
    if(config.componentName){
      if(config.componentName.prepend){
        sourceName = config.componentName.prepend + sourceName;
      }
      if(config.componentName.append){
        sourceName = sourceName + config.componentName.append;
      }
    }
    if(config.base){
      if(folderInput){
        folderInput = path.join(config.base, folderInput);
      }
      else{
        sourceName = path.join(config.base, sourceName);
      }
    }
    if(config.styleExtension)
      stylesExt = config.styleExtension
  }


  let basename = path.basename(sourceName);
  let dir  = folderInput ? path.resolve(path.join(folderInput, sourceName)) : path.resolve(sourceName);
  let styles = path.resolve(dir, basename + "." + stylesExt);
  let js = path.resolve(dir, basename + ".js");
  let index = path.resolve(dir, "index.js");

  let jsContent = require("./templates/component")({basename});
  let actionContent = require("./templates/action")({basename});
  let reducerContent = require("./templates/reducer")({basename});

  if(typeof config !== "undefined" && typeof config.template !== "undefined"){
    if(config.template.component){
      jsContent = require( path.resolve(process.cwd(), config.template.component) )({basename});
    }
    if(config.template.action){
      actionContent = require( path.resolve(process.cwd(), config.template.action) )({basename});
    }
    if(config.template.reducer){
      reducerContent = require( path.resolve(process.cwd(), config.template.reducer) )({basename});
    }
  }


  let indexContent = `import ${basename} from "./${basename}";
export default ${basename};
`;

  let actionAndReducer = (type) => {

    let _content = type === TYPE.ACTION ? actionContent : reducerContent;
    let _path = '';


    let resultPath = '';
    if(folderInput)
      resultPath = path.join(folderInput);
    if(typeof config !== "undefined"){
      _path = type === TYPE.ACTION ? config.actionsDir : config.reducersDir;

      if(config.base){
        resultPath = path.join(config.base, path.dirname(name) || '');
      }
      if(_path){
        resultPath = path.join(config.base || '', _path, path.dirname(name) || '');
      }

    }
    if(fs.existsSync(path.resolve(path.join(resultPath, path.basename(name))) + ".js")){
      console.error(`${chalk.yellow(name)} ${chalk.red('already exists')}`);
      return false;
    }

    try{
      fs.writeSync(fs.openSync( path.resolve(path.join(resultPath, path.basename(name))) + ".js", "w"), _content);
    }catch(e){
      if(e.code === 'ENOENT'){
        mkdirSync(path.resolve(resultPath));
        fs.writeSync(fs.openSync( path.resolve(path.join(resultPath, path.basename(name))) + ".js", "w"), _content);
      }
    }

    console.log(`${chalk.blue(type === TYPE.ACTION ? 'Action' : 'Reducer')}  ${chalk.yellow(name)} successfully created`);
  };

  if(options.reduxAction){
   actionAndReducer(TYPE.ACTION)
  }else if(options.reduxReducer){
    actionAndReducer(TYPE.REDUCER)
  }
  else {
    try {
      if(fs.existsSync(`./${folderInput ? path.join(folderInput, sourceName) : sourceName}`)){
        console.error(`${chalk.yellow(name)} ${chalk.red('already exists')}`);
        return false;
      }
      else{
        mkdirSync(`./${folderInput? path.join(folderInput, sourceName) : sourceName}`)
      }

    } catch(e) {
      if(e.code === 'EEXIST'){
        console.log(`${chalk.yellow(sourceName)} ${chalk.red('already exists')}`);
      }else{
        console.error(e);
      }
      return false;
    }
    fs.openSync(styles, "w");
    fs.writeSync(fs.openSync(js, "w"), jsContent);
    fs.writeSync(fs.openSync(index, "w"), indexContent);
    console.log(`${chalk.blue('Component:')} ${chalk.yellow(name)} successfully created`);
  }
}

program
  .version('1.0.0')
  .on('--help', function(){
    console.log('  Examples:');
    console.log('');
    console.log('    $ rch -c Home');
    console.log('    $ rch -c src/myFolder/Login');
    console.log('    $ rch auth -r');
    console.log('    $ rch actions/auth -a');
    console.log('');
  })
  .option('-s, --styles', 'styles extension [default: scss]')
  .option('-c, --redux-component', 'create react component with redux')
  .option('-r, --redux-reducer', 'create redux reducer')
  .option('-a, --redux-action', 'create redux action')
  .arguments('<name>')
  .action(run)
  .parse(process.argv);
